@@ -4,19 +4,22 @@ use strict;
 use warnings;
 use warnings::register;
 use Carp;
+use POSIX; # Only needed for max integer size check in large ranges
 
 require Exporter;
 
 our @ISA = qw(Exporter);
 
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 sub new {
   my $this = shift;
   my $class = ref($this) || $this;
   my $self = {};
   bless $self, $class;
+  # Max size of range before its stored as a pointer instead of hashed
+  $self->{max_hash_size} = 1000;
   $self->initialize("add", @_);
   return $self;
 }
@@ -59,10 +62,18 @@ sub initialize {
         }
         else {
           if ($type eq "add") {
-            $self->_addnumbers($start .. $end);
+            if(($end - $start) > $self->{max_hash_size}) {
+                $self->_addrange($start, $end);
+            } else {
+                $self->_addnumbers($start .. $end);
+            }
           }
           elsif ($type eq "del") {
-            $self->_delnumbers($start .. $end);
+            if($end - $start > $self->{max_hash_size}) {
+                $self->_delrange($start, $end);
+            } else {
+                $self->_delnumbers($start .. $end);
+            }
           }
           else {
             die "Neither 'add' nor 'del' was passed initialize()";
@@ -84,6 +95,43 @@ sub initialize {
   }
 }
 
+sub set_max_hash_size {
+    my $self = shift;
+    my $val = shift;
+    if($val !~ m/^\d+$/) { return 0; }
+    $self->{max_hash_size} = $val;
+}
+
+sub _addrange {
+    my $self = shift;
+    my $start = shift;
+    my $end = shift;
+    $self->{_largeRangehash}{"$start .. $end"} = [$start, $end];
+}
+
+sub _delrange {
+    my $self = shift;
+    my $start = shift;
+    my $end = shift;
+    delete $self->{_largeRangehash}{"$start .. $end"};
+}
+
+sub _testlarge {
+    my $self = shift;
+    my $test = shift;
+    if(!exists($self->{_largeRangehash})) {
+        return 0;
+    }
+    foreach my $rangeID (keys(%{$self->{_largeRangehash}})) {
+      my $range = $self->{_largeRangehash}->{$rangeID};
+      if ($test >= @$range[0]
+        && $test <= @$range[1]) {
+        return 1;
+      }
+    }
+    return 0;
+}
+
 sub _addnumbers {
   my $self = shift;
   foreach my $number (@_) {
@@ -107,7 +155,12 @@ sub _delnumbers {
 sub inrange {
   my $self   = shift;
   if (scalar(@_) == 1) {
-    return (exists($self->{_rangehash}{-+-$_[0]})) ? 1 : 0;
+    if ( exists($self->{_rangehash}{$_[0]})
+        || $self->_testlarge($_[0])) {
+        return 1;
+    } else {
+        return 0;
+    }
   } else {
     if (wantarray) {
       my @returncodes;
@@ -140,6 +193,19 @@ sub range {
   my $self = shift;
   if (wantarray) {
     my @range = keys(%{$self->{_rangehash}});
+    if(exists($self->{_largeRangehash})) {
+        foreach my $rangeID (keys(%{$self->{_largeRangehash}})) {
+          my $range = $self->{_largeRangehash}->{$rangeID};
+            if ( @$range[0] > LONG_MAX
+                || @$range[1] > LONG_MAX
+                || ( @$range[1] -  @$range[0]) > LONG_MAX ) {
+                carp "Range to large to return" if (warnings::enabled());
+                return 0;
+            }
+            
+            @range = (@range, @$range[0]..@$range[1]);
+        }
+    }
     my @sorted = sort {$a <=> $b} @range;
     return @sorted;
   }
@@ -163,8 +229,15 @@ sub range {
 
 sub size {
   my $self = shift;
-  my @temp = $self->range;
-  return scalar(@temp);
+  my @temp = keys(%{$self->{_rangehash}});;
+  my $size = scalar(@temp);
+  if(exists($self->{_largeRangehash})) {
+    foreach my $rangeID (keys(%{$self->{_largeRangehash}})) {
+      my $range = $self->{_largeRangehash}->{$rangeID};
+      $size += (@$range[1] - @$range[0]) + 1;
+    }
+  }  
+  return $size;
 }
 
 1;
@@ -267,7 +340,7 @@ Larry Shatzer, Jr., E<lt>larrysh@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2004-8 by Larry Shatzer, Jr.
+Copyright (C) 2004-12 by Larry Shatzer, Jr.
 
 This library is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
